#include "peripheralwidget.h"

peripheralWidget::peripheralWidget(const QString &name, QWidget *parent) :
    QWidget(parent)
{
    p_name = name;
    p_timer = new QTimer(this);
    p_timer->setInterval(500);
    for(int i=0;i<10;i++)
    {
        registersWdgts.append(new registerWidget(QString("REG%1").arg(i),i*4));
        registersWdgts.at(i)->setValue((i<<24)+(i<<8)+i);
        connect(registersWdgts.at(i),SIGNAL(repaint()),this,SLOT(repaint()));
    }
    setAttribute(Qt::WA_AlwaysShowToolTips);
    setMouseTracking(true);
    setFocusPolicy(Qt::StrongFocus);
    selectedReg = -1;
    connect(p_timer,SIGNAL(timeout()),this,SLOT(blinkCursor()));
    setFont(QFont("Utopia", 14,QFont::Bold));
}

void peripheralWidget::blinkCursor()
{
    if(selectedReg!=-1)
        registersWdgts.at(selectedReg)->blinkCursor();
    repaint();
}

void peripheralWidget::mousePressEvent(QMouseEvent *event)
{
    p_timer->stop();
    if(selectedReg!=-1)
        registersWdgts.at(selectedReg)->leave();
    for(int i=0; i<registersWdgts.count();i++)
    {
        if(registersWdgts.at(i)->contains(event->pos()))
        {
            registersWdgts.at(i)->enter(registersWdgts.at(i)->cursorIndex(event->pos().x()));
            selectedReg = i;
            p_timer->start();
        }
    }
    repaint();
}

void peripheralWidget::mouseMoveEvent(QMouseEvent *event)
{

}

void peripheralWidget::mouseReleaseEvent(QMouseEvent *event)
{

}

void peripheralWidget::keyPressEvent(QKeyEvent *event)
{
    if(this->selectedReg!=-1){
        if(event->modifiers()==Qt::ControlModifier)
        {
            switch(event->key())
            {
            case Qt::Key_Up:
                registersWdgts.at(selectedReg)->set(registersWdgts.at(selectedReg)->cursorIndex());
                break;
            case Qt::Key_Down:
                registersWdgts.at(selectedReg)->clear(registersWdgts.at(selectedReg)->cursorIndex());
                break;
            default:
                break;
            }
        }
        else
        {
            switch(event->key())
            {
            case Qt::Key_0:
                registersWdgts.at(selectedReg)->clear(registersWdgts.at(selectedReg)->cursorIndex());
                registersWdgts.at(selectedReg)->moveCursorRight(1);
                break;
            case Qt::Key_1:
                registersWdgts.at(selectedReg)->set(registersWdgts.at(selectedReg)->cursorIndex());
                registersWdgts.at(selectedReg)->moveCursorRight(1);
                break;
            case Qt::Key_Right:
                registersWdgts.at(selectedReg)->moveCursorRight(1);
                this->repaint();
                break;
            case Qt::Key_Left:
                registersWdgts.at(selectedReg)->moveCursorLeft(1);
                this->repaint();
                break;
            case Qt::Key_Up:
                up();
                break;
            case Qt::Key_Down:
                down();
                break;
            default:
                break;
            }
        }

    }
}

void peripheralWidget::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)
    QPainter painter(this);
    QPoint offset=QPoint(0,0);
    int nameWidth = fontMetrics().width(p_name);
    painter.drawText((this->minimumWidth()/2)-nameWidth,4,fontMetrics().width(p_name),fontMetrics().height()+4,Qt::AlignCenter,p_name);
    offset+=QPoint(0,fontMetrics().height()+8);
    for(int i=0;i<registersWdgts.count();i++)
    {
        offset = registersWdgts.at(i)->paint(&painter,offset);
    }
    setMinimumSize(registersWdgts.first()->boundingRect().width(),offset.y());
    updateGeometry();
}

void peripheralWidget::up()
{
    if(selectedReg!=-1 && selectedReg >0)
    {
        registersWdgts.at(selectedReg)->leave();
        selectedReg-=1;
        registersWdgts.at(selectedReg)->enter(registersWdgts.at(selectedReg+1)->cursorIndex());
        repaint();
    }
}

void peripheralWidget::down()
{
    if(selectedReg!=-1 && selectedReg <(registersWdgts.count()-1))
    {
        registersWdgts.at(selectedReg)->leave();
        selectedReg+=1;
        registersWdgts.at(selectedReg)->enter(registersWdgts.at(selectedReg-1)->cursorIndex());
        repaint();
    }
}
